from xpresso.binders._body.json.extractors import (
    BodyExtractorMarker,
    FieldExtractorMarker,
    SupportsJsonDecoder,
)
from xpresso.binders._body.json.openapi import BodyOpenAPIMarker, FieldOpenAPIMarker

__all__ = (
    "SupportsJsonDecoder",
    "BodyExtractorMarker",
    "FieldExtractorMarker",
    "BodyOpenAPIMarker",
    "FieldOpenAPIMarker",
)



from xpresso.binders._body.discriminated.extractors import BodyExtractorMarker
from xpresso.binders._body.discriminated.openapi import BodyOpenAPIMarker

__all__ = (
    "BodyExtractorMarker",
    "BodyOpenAPIMarker",
)



import inspect
import typing

from pydantic.fields import ModelField
from pydantic.schema import get_flat_models_from_field

from xpresso._utils.schemas import openapi_schema_from_pydantic_field
from xpresso._utils.typing import model_field_from_param
from xpresso.binders._utils.examples import parse_examples
from xpresso.binders.api import (
    ModelNameMap,
    Schemas,
    SupportsOpenAPIBody,
    SupportsOpenAPIField,
)
from xpresso.openapi import models as openapi_models


class _BodyOpenAPI(typing.NamedTuple):
    description: typing.Optional[str]
    examples: typing.Optional[openapi_models.Examples]
    field: ModelField
    required: bool
    include_in_schema: bool

    def get_models(self) -> typing.List[type]:
        return list(get_flat_models_from_field(self.field, set()))

    def get_openapi_body(
        self, model_name_map: ModelNameMap, schemas: Schemas
    ) -> openapi_models.RequestBody:
        return openapi_models.RequestBody(
            description=self.description,
            required=self.required,
            content={
                "application/json": openapi_models.MediaType(
                    schema=openapi_schema_from_pydantic_field(
                        self.field, model_name_map, schemas
                    ),
                    examples=self.examples,
                )
            },
        )


class _FieldOpenAPI(typing.NamedTuple):
    field: ModelField

    def get_models(self) -> typing.List[type]:
        return list(get_flat_models_from_field(self.field, set()))

    def get_field_schema(
        self, model_name_map: ModelNameMap, schemas: Schemas
    ) -> openapi_models.Schema:
        return openapi_schema_from_pydantic_field(self.field, model_name_map, schemas)

    def get_field_encoding(
        self, model_name_map: ModelNameMap, schemas: Schemas
    ) -> openapi_models.Encoding:
        return openapi_models.Encoding(contentType="application/json")


class BodyOpenAPIMarker(typing.NamedTuple):
    description: typing.Optional[str]
    examples: typing.Optional[
        typing.Dict[str, typing.Union[openapi_models.Example, typing.Any]]
    ]
    include_in_schema: bool

    def register_parameter(self, param: inspect.Parameter) -> SupportsOpenAPIBody:
        examples = parse_examples(self.examples) if self.examples else None
        field = model_field_from_param(param)
        required = field.required is not False
        return _BodyOpenAPI(
            description=self.description,
            examples=examples,
            field=field,
            required=required,
            include_in_schema=self.include_in_schema,
        )


class FieldOpenAPIMarker(typing.NamedTuple):
    def register_parameter(self, param: inspect.Parameter) -> SupportsOpenAPIField:
        return _FieldOpenAPI(model_field_from_param(param))



from xpresso.binders._body.forms.field_extractors import FieldExtractorMarker
from xpresso.binders._body.forms.field_openapi import FieldOpenAPIMarker
from xpresso.binders._body.forms.form_data_extractors import BodyExtractorMarker
from xpresso.binders._body.forms.form_data_openapi import BodyOpenAPIMarker
from xpresso.binders._body.forms.form_encoded_extractor import (
    FormEncodedFieldExtractorMarker,
)
from xpresso.binders._body.forms.form_encoded_openapi import FormEncodedOpenAPIMarker

__all__ = (
    "BodyExtractorMarker",
    "BodyOpenAPIMarker",
    "FormEncodedFieldExtractorMarker",
    "FormEncodedOpenAPIMarker",
    "FieldExtractorMarker",
    "FieldOpenAPIMarker",
)



import inspect
import typing

from di.api.dependencies import CacheKey, DependantBase
from di.dependant import Dependant, Marker

from xpresso._utils.compat import Protocol
from xpresso.binders.api import (
    SupportsBodyExtractor,
    SupportsExtractor,
    SupportsFieldExtractor,
    SupportsOpenAPIBody,
    SupportsOpenAPIField,
    SupportsOpenAPIParameter,
)

T = typing.TypeVar("T", covariant=True)


class SupportsMarker(Protocol[T]):
    def register_parameter(self, param: inspect.Parameter) -> T:
        ...


class ParameterBinder(Dependant[typing.Any]):
    def __init__(
        self,
        in_: str,
        openapi: SupportsOpenAPIParameter,
        extractor: SupportsExtractor,
    ):
        self.in_ = in_
        self.openapi = openapi
        super().__init__(call=extractor.extract, scope="connection")

    @property
    def cache_key(self) -> CacheKey:
        # As per the spec, parameters are identified
        # by name and location
        return (self.__class__, self.openapi.name, self.in_)


class ParameterBinderMarker(Marker):
    def __init__(
        self,
        *,
        in_: str,
        extractor_marker: SupportsMarker[SupportsExtractor],
        openapi_marker: SupportsMarker[SupportsOpenAPIParameter],
    ) -> None:
        self.in_ = in_
        self.extractor_marker = extractor_marker
        self.openapi_marker = openapi_marker

    def register_parameter(self, param: inspect.Parameter) -> DependantBase[typing.Any]:
        return ParameterBinder(
            in_=self.in_,
            openapi=self.openapi_marker.register_parameter(param),
            extractor=self.extractor_marker.register_parameter(param),
        )


class BodyBinder(Dependant[typing.Any]):
    def __init__(
        self,
        *,
        openapi: typing.Optional[SupportsOpenAPIBody],
        extractor: SupportsBodyExtractor,
    ) -> None:
        super().__init__(call=extractor.extract, scope="connection")
        self.openapi = openapi
        self.extractor = extractor

    @property
    def cache_key(self) -> CacheKey:
        # Treat each body as unique since there should only be one body
        return (self.__class__, id(self))


class BodyBinderMarker(Marker):
    def __init__(
        self,
        *,
        extractor_marker: typing.Optional[SupportsMarker[SupportsBodyExtractor]],
        field_extractor_marker: typing.Optional[SupportsMarker[SupportsFieldExtractor]],
        openapi_marker: typing.Optional[SupportsMarker[SupportsOpenAPIBody]],
        openapi_field_marker: typing.Optional[SupportsMarker[SupportsOpenAPIField]],
    ) -> None:
        self.field_extractor_marker = field_extractor_marker
        self.extractor_marker = extractor_marker
        self.openapi_marker = openapi_marker
        self.openapi_field_marker = openapi_field_marker

    def register_parameter(self, param: inspect.Parameter) -> BodyBinder:
        openapi = (
            None
            if self.openapi_marker is None
            else self.openapi_marker.register_parameter(param)
        )
        if self.extractor_marker is None:
            raise TypeError("Top-level bodies MUST provide an extractor implementation")
        return BodyBinder(
            openapi=openapi,
            extractor=self.extractor_marker.register_parameter(param),
        )



import inspect
import typing

from pydantic.error_wrappers import ErrorWrapper
from starlette.datastructures import FormData

from xpresso._utils.typing import model_field_from_param
from xpresso.binders._body.form_field import SupportsXpressoFormDataFieldExtractor
from xpresso.binders._utils.forms import Extractor as FormExtractor
from xpresso.binders._utils.forms import UnexpectedFileReceived, get_extractor
from xpresso.binders.exceptions import InvalidSerialization
from xpresso.exceptions import RequestValidationError
from xpresso.typing import Some


class _FormEncodedFieldExtractor(typing.NamedTuple):
    alias: typing.Optional[str]
    style: str
    explode: bool
    field_name: str
    extractor: FormExtractor

    async def extract_from_form(
        self, form: FormData, *, loc: typing.Iterable[typing.Union[int, str]]
    ) -> typing.Optional[Some]:
        try:
            return self.extractor(name=self.field_name, params=form.multi_items())
        except InvalidSerialization as e:
            raise RequestValidationError(
                [
                    ErrorWrapper(
                        exc=TypeError("Data is not a valid URL encoded form"),
                        loc=tuple((*loc, self.field_name)),
                    )
                ]
            ) from e
        except UnexpectedFileReceived as exc:
            raise RequestValidationError(
                [
                    ErrorWrapper(
                        exc=exc,
                        loc=tuple((*loc, self.field_name)),
                    )
                ]
            ) from exc


class FormEncodedFieldExtractorMarker(typing.NamedTuple):
    alias: typing.Optional[str]
    style: str
    explode: bool

    def register_parameter(
        self, param: inspect.Parameter
    ) -> SupportsXpressoFormDataFieldExtractor:
        field_name = self.alias or param.name
        extractor = get_extractor(
            style=self.style, explode=self.explode, field=model_field_from_param(param)
        )
        return _FormEncodedFieldExtractor(
            alias=self.alias,
            style=self.style,
            explode=self.explode,
            field_name=field_name,
            extractor=extractor,
        )



import inspect
import typing

from di.typing import get_markers_from_annotation

from xpresso._utils.typing import model_field_from_param
from xpresso.binders._body.form_field import SupportsXpressoFormDataFieldOpenAPI
from xpresso.binders.api import ModelNameMap, Schemas, SupportsOpenAPIField
from xpresso.binders.dependants import BodyBinderMarker
from xpresso.openapi import models


class _Base(typing.NamedTuple):
    field_name: str
    include_in_schema: bool
    field_openapi: SupportsOpenAPIField

    def get_models(self) -> typing.List[type]:
        return self.field_openapi.get_models()

    def get_field_encoding(
        self, model_name_map: ModelNameMap, schemas: Schemas
    ) -> models.Encoding:
        return self.field_openapi.get_field_encoding(model_name_map, schemas)


class _FieldOpenAPI(_Base):
    def get_field_schema(
        self, model_name_map: ModelNameMap, schemas: Schemas
    ) -> models.Schema:
        return self.field_openapi.get_field_schema(
            model_name_map=model_name_map, schemas=schemas
        )


class _RepeatedFieldOpenAPI(_Base):
    def get_field_schema(
        self, model_name_map: ModelNameMap, schemas: Schemas
    ) -> models.Schema:
        return models.Schema(
            type="array",
            items=self.field_openapi.get_field_schema(
                model_name_map=model_name_map, schemas=schemas
            ),
        )


class FieldOpenAPIMarker(typing.NamedTuple):
    alias: typing.Optional[str]
    include_in_schema: bool
    repeated: bool

    def register_parameter(
        self, param: inspect.Parameter
    ) -> SupportsXpressoFormDataFieldOpenAPI:
        marker = next(
            get_markers_from_annotation(param.annotation, BodyBinderMarker), None
        )
        if marker is None:
            raise TypeError(
                "No field marker found"
                "\n You must include a valid field marker using ExtractField[AsJson[...]]"
                " or Annotated[..., Json(), Field()]"
            )
        if marker.openapi_field_marker is None:
            raise TypeError(f"The field {param.name} is not valid as a form field")
        openapi_provider = marker.openapi_field_marker.register_parameter(param)
        field_name = self.alias or model_field_from_param(param).alias
        if self.repeated:
            return _RepeatedFieldOpenAPI(
                field_name=field_name,
                field_openapi=openapi_provider,
                include_in_schema=self.include_in_schema,
            )
        return _FieldOpenAPI(
            field_name=field_name,
            field_openapi=openapi_provider,
            include_in_schema=self.include_in_schema,
        )



import inspect
import typing

from starlette.datastructures import FormData

import xpresso.openapi.models as openapi_models
from xpresso._utils.compat import Protocol
from xpresso.binders.api import ModelNameMap, Schemas
from xpresso.typing import Some


class SupportsXpressoFormDataFieldOpenAPI(Protocol):
    @property
    def field_name(self) -> str:
        ...

    @property
    def include_in_schema(self) -> bool:
        ...

    def get_models(self) -> typing.Iterable[type]:
        ...

    def get_field_encoding(
        self, model_name_map: ModelNameMap, schemas: Schemas
    ) -> openapi_models.Encoding:
        ...

    def get_field_schema(
        self, model_name_map: ModelNameMap, schemas: Schemas
    ) -> openapi_models.Schema:
        ...


class SupportsXpressoFormDataFieldOpenAPIMarker(Protocol):
    def register_parameter(
        self, param: inspect.Parameter
    ) -> SupportsXpressoFormDataFieldOpenAPI:
        ...


class SupportsXpressoFormDataFieldExtractor(Protocol):
    @property
    def field_name(self) -> str:
        ...

    async def extract_from_form(
        self, form: FormData, *, loc: typing.Iterable[typing.Union[int, str]]
    ) -> typing.Optional[Some]:
        ...


class SupportsXpressoFormDataFieldExtractorMarker(Protocol):
    def register_parameter(
        self, param: inspect.Parameter
    ) -> SupportsXpressoFormDataFieldExtractor:
        ...


class FormFieldMarker(typing.NamedTuple):
    openapi_marker: SupportsXpressoFormDataFieldOpenAPIMarker
    extractor_marker: SupportsXpressoFormDataFieldExtractorMarker



import inspect
import typing

from pydantic.error_wrappers import ErrorWrapper
from pydantic.fields import ModelField
from starlette.datastructures import UploadFile
from starlette.requests import HTTPConnection, Request

from xpresso._utils.compat import Protocol
from xpresso._utils.typing import model_field_from_param
from xpresso.binders._body.media_type_validator import MediaTypeValidator
from xpresso.binders._body.media_type_validator import (
    get_validator as get_media_type_validator,
)
from xpresso.binders._body.pydantic_field_validator import validate_body_field
from xpresso.binders._utils.stream_to_bytes import convert_stream_to_bytes
from xpresso.binders.api import SupportsBodyExtractor, SupportsFieldExtractor
from xpresso.exceptions import RequestValidationError
from xpresso.typing import Some


class SupportsJsonDecoder(Protocol):
    def __call__(self, s: typing.Union[str, bytes]) -> typing.Any:
        ...


def _decode(
    decoder: SupportsJsonDecoder,
    value: typing.Union[str, bytes],
    loc: typing.Iterable[typing.Union[int, str]],
) -> typing.Union[bytes, UploadFile]:
    try:
        decoded = decoder(value)
    except Exception as e:
        raise RequestValidationError(
            [
                ErrorWrapper(
                    exc=TypeError("Data is not valid JSON"),
                    loc=tuple(loc),
                )
            ]
        ) from e
    return decoded


class _BodyExtractor(typing.NamedTuple):
    field: ModelField
    decoder: SupportsJsonDecoder
    media_type_validator: MediaTypeValidator
    consume: bool

    def matches_media_type(self, media_type: typing.Optional[str]) -> bool:
        return self.media_type_validator.matches(media_type)

    async def extract(self, connection: HTTPConnection) -> typing.Any:
        assert isinstance(connection, Request)
        loc = ("body",)
        self.media_type_validator.validate(
            connection.headers.get("content-type", None), loc=loc
        )
        if self.consume:
            data_from_stream = await convert_stream_to_bytes(connection.stream())
            if data_from_stream is None:
                return validate_body_field(None, field=self.field, loc=loc)
        else:
            data_from_stream = await connection.body()
        return validate_body_field(
            Some(_decode(self.decoder, data_from_stream, loc=loc)),
            field=self.field,
            loc=loc,
        )


class _FieldExtractor(typing.NamedTuple):
    decoder: SupportsJsonDecoder

    async def extract_from_field(
        self,
        field: typing.Union[str, UploadFile],
        *,
        loc: typing.Iterable[typing.Union[str, int]],
    ) -> typing.Any:
        if isinstance(field, UploadFile):
            return _decode(self.decoder, await field.read(), loc=loc)
        return _decode(self.decoder, field, loc=loc)


class BodyExtractorMarker(typing.NamedTuple):
    decoder: SupportsJsonDecoder
    enforce_media_type: bool
    consume: bool

    def register_parameter(self, param: inspect.Parameter) -> SupportsBodyExtractor:
        if self.enforce_media_type:
            media_type_validator = get_media_type_validator("application/json")
        else:
            media_type_validator = get_media_type_validator(None)
        return _BodyExtractor(
            field=model_field_from_param(param),
            decoder=self.decoder,
            media_type_validator=media_type_validator,
            consume=self.consume,
        )


class FieldExtractorMarker(typing.NamedTuple):
    decoder: SupportsJsonDecoder

    def register_parameter(self, param: inspect.Parameter) -> SupportsFieldExtractor:
        return _FieldExtractor(self.decoder)



import inspect
import typing

from di.typing import get_markers_from_annotation
from starlette.datastructures import FormData

from xpresso._utils.typing import model_field_from_param
from xpresso.binders._body.form_field import SupportsXpressoFormDataFieldExtractor
from xpresso.binders.api import SupportsFieldExtractor as SupportsFieldExtractor
from xpresso.binders.dependants import BodyBinderMarker
from xpresso.typing import Some


class _FieldExtractor(typing.NamedTuple):
    field_name: str
    field_extractor: SupportsFieldExtractor

    async def extract_from_form(
        self, form: FormData, *, loc: typing.Iterable[typing.Union[int, str]]
    ) -> typing.Optional[Some]:
        if self.field_name not in form:
            return None
        return Some(
            await self.field_extractor.extract_from_field(
                form[self.field_name], loc=(*loc, self.field_name)
            )
        )


class _RepeatedFieldExtractor(typing.NamedTuple):
    field_name: str
    field_extractor: SupportsFieldExtractor

    async def extract_from_form(
        self, form: FormData, *, loc: typing.Iterable[typing.Union[int, str]]
    ) -> typing.Optional[Some]:
        return Some(
            [
                await self.field_extractor.extract_from_field(
                    val, loc=(*loc, self.field_name)
                )
                for field_name, val in form.multi_items()
                if field_name == self.field_name
            ]
        )


class FieldExtractorMarker(typing.NamedTuple):
    alias: typing.Optional[str]
    repeated: bool

    def register_parameter(
        self, param: inspect.Parameter
    ) -> SupportsXpressoFormDataFieldExtractor:
        field = model_field_from_param(param)
        field_name = self.alias or field.alias
        marker = next(
            get_markers_from_annotation(param.annotation, BodyBinderMarker), None
        )
        if marker is None:
            raise TypeError(
                "No field marker found"
                "\n You must include a valid field marker using ExtractField[AsJson[...]]"
                " or Annotated[..., Json(), Field()]"
            )
        if marker.field_extractor_marker is None:
            raise TypeError(f"The field {param.name} is not valid as a form field")
        field_extractor = marker.field_extractor_marker.register_parameter(param)
        if self.repeated:
            return _RepeatedFieldExtractor(
                field_name=field_name, field_extractor=field_extractor
            )
        else:
            return _FieldExtractor(
                field_name=field_name, field_extractor=field_extractor
            )



from xpresso.binders._body.files.extractors import (
    BodyExtractorMarker,
    FieldExtractorMarker,
)
from xpresso.binders._body.files.openapi import BodyOpenAPIMarker, FieldOpenAPIMarker

__all__ = (
    "BodyExtractorMarker",
    "FieldExtractorMarker",
    "BodyOpenAPIMarker",
    "FieldOpenAPIMarker",
)



import typing
from json import loads as json_loads

import xpresso.binders._body.discriminated as discriminated
import xpresso.binders._body.files as files
import xpresso.binders._body.forms as forms
import xpresso.binders._body.json as json
import xpresso.binders.dependants as param_dependants
import xpresso.openapi.models as openapi_models
from xpresso._utils.compat import Annotated, Literal
from xpresso.binders._body.form_field import FormFieldMarker

Example = typing.Union[openapi_models.Example, typing.Any]


def Json(
    *,
    examples: typing.Optional[typing.Dict[str, Example]] = None,
    description: typing.Optional[str] = None,
    decoder: json.SupportsJsonDecoder = json_loads,
    enforce_media_type: bool = True,
    consume: bool = True,
    include_in_schema: bool = True,
) -> param_dependants.BodyBinderMarker:
    body_extractor_marker = json.BodyExtractorMarker(
        decoder=decoder,
        enforce_media_type=enforce_media_type,
        consume=consume,
    )
    field_extractor_marker = json.FieldExtractorMarker(decoder=decoder)
    body_openapi_marker = json.BodyOpenAPIMarker(
        description=description,
        examples=examples,
        include_in_schema=include_in_schema,
    )
    openapi_field_marker = json.FieldOpenAPIMarker()
    return param_dependants.BodyBinderMarker(
        extractor_marker=body_extractor_marker,
        field_extractor_marker=field_extractor_marker,
        openapi_marker=body_openapi_marker,
        openapi_field_marker=openapi_field_marker,
    )


def File(
    *,
    media_type: typing.Optional[str] = None,
    enforce_media_type: bool = True,
    examples: typing.Optional[typing.Dict[str, Example]] = None,
    description: typing.Optional[str] = None,
    format: Literal["binary", "base64"] = "binary",
    consume: bool = True,
    include_in_schema: bool = True,
) -> param_dependants.BodyBinderMarker:
    body_extractor = files.BodyExtractorMarker(
        media_type=media_type,
        enforce_media_type=enforce_media_type,
        consume=consume,
    )
    field_extractor_marker = files.FieldExtractorMarker()
    openapi_body_marker = files.BodyOpenAPIMarker(
        description=description,
        examples=examples,
        media_type=media_type,
        format=format,
        include_in_schema=include_in_schema,
    )
    openapi_field_marker = files.FieldOpenAPIMarker(
        media_type=media_type,
        format=format,
    )
    return param_dependants.BodyBinderMarker(
        extractor_marker=body_extractor,
        field_extractor_marker=field_extractor_marker,
        openapi_marker=openapi_body_marker,
        openapi_field_marker=openapi_field_marker,
    )


def FormEncodedField(
    *,
    alias: typing.Optional[str] = None,
    style: openapi_models.FormDataStyles = "form",
    explode: bool = True,
    include_in_schema: bool = True,
) -> FormFieldMarker:
    extractor_marker = forms.FormEncodedFieldExtractorMarker(
        alias=alias,
        style=style,
        explode=explode,
    )
    openapi_marker = forms.FormEncodedOpenAPIMarker(
        alias=alias,
        style=style,
        explode=explode,
        include_in_schema=include_in_schema,
    )
    return FormFieldMarker(
        extractor_marker=extractor_marker,
        openapi_marker=openapi_marker,
    )


def FormField(
    *,
    alias: typing.Optional[str] = None,
    include_in_schema: bool = True,
) -> FormFieldMarker:
    extractor = forms.FieldExtractorMarker(alias=alias, repeated=False)
    openapi = forms.FieldOpenAPIMarker(
        alias=alias,
        include_in_schema=include_in_schema,
        repeated=False,
    )
    return FormFieldMarker(
        extractor_marker=extractor,
        openapi_marker=openapi,
    )


def RepeatedFormField(
    *,
    alias: typing.Optional[str] = None,
    include_in_schema: bool = True,
) -> FormFieldMarker:
    extractor = forms.FieldExtractorMarker(alias=alias, repeated=True)
    openapi = forms.FieldOpenAPIMarker(
        alias=alias,
        include_in_schema=include_in_schema,
        repeated=True,
    )
    return FormFieldMarker(
        extractor_marker=extractor,
        openapi_marker=openapi,
    )


def Form(
    *,
    enforce_media_type: bool = True,
    examples: typing.Optional[typing.Dict[str, Example]] = None,
    description: typing.Optional[str] = None,
    include_in_schema: bool = True,
) -> param_dependants.BodyBinderMarker:
    body_extractor_marker = forms.BodyExtractorMarker(
        media_type="application/x-www-form-urlencoded",
        enforce_media_type=enforce_media_type,
    )
    body_openapi_marker = forms.BodyOpenAPIMarker(
        description=description,
        examples=examples,
        media_type="application/x-www-form-urlencoded",
        include_in_schema=include_in_schema,
    )
    return param_dependants.BodyBinderMarker(
        extractor_marker=body_extractor_marker,
        field_extractor_marker=None,
        openapi_marker=body_openapi_marker,
        openapi_field_marker=None,
    )


def Multipart(
    *,
    enforce_media_type: bool = True,
    examples: typing.Optional[typing.Dict[str, Example]] = None,
    description: typing.Optional[str] = None,
    include_in_schema: bool = True,
) -> param_dependants.BodyBinderMarker:
    body_extractor_marker = forms.BodyExtractorMarker(
        media_type="multipart/form-data",
        enforce_media_type=enforce_media_type,
    )
    body_openapi_marker = forms.BodyOpenAPIMarker(
        description=description,
        examples=examples,
        media_type="multipart/form-data",
        include_in_schema=include_in_schema,
    )
    return param_dependants.BodyBinderMarker(
        extractor_marker=body_extractor_marker,
        field_extractor_marker=None,
        openapi_marker=body_openapi_marker,
        openapi_field_marker=None,
    )


def ContentTypeDiscriminatedBody(
    *,
    description: typing.Optional[str] = None,
) -> param_dependants.BodyBinderMarker:
    extractor = discriminated.BodyExtractorMarker()
    openapi = discriminated.BodyOpenAPIMarker(
        description=description,
    )
    return param_dependants.BodyBinderMarker(
        extractor_marker=extractor,
        field_extractor_marker=None,
        openapi_marker=openapi,
        openapi_field_marker=None,
    )


# Convenience type aliases
_T = typing.TypeVar("_T")
FromJson = Annotated[_T, Json()]
FromFile = Annotated[_T, File()]
FromFormField = Annotated[_T, FormEncodedField()]
ExtractField = Annotated[_T, FormField()]
ExtractRepeatedField = Annotated[_T, RepeatedFormField()]
FromFormData = Annotated[_T, Form()]
FromMultipart = Annotated[_T, Multipart()]
ByContentType = Annotated[_T, ContentTypeDiscriminatedBody()]
